@@ -182,7 +182,7 @@ publish_to_all_am_prepared_cb (GObject *source_object,
   GList *accounts, *l;
   GError *error = NULL;
 
-  if (!tp_account_manager_prepare_finish (manager, result, &error))
+  if (!tp_proxy_prepare_finish (manager, result, &error))
     {
       DEBUG ("Failed to prepare account manager: %s", error->message);
       g_error_free (error);
@@ -214,7 +214,7 @@ publish_to_all_connections (EmpathyLocationManager *self,
   data->self = g_object_ref (self);
   data->force_publication = force_publication;
 
-  tp_account_manager_prepare_async (self->priv->account_manager, NULL,
+  tp_proxy_prepare_async (self->priv->account_manager, NULL,
       publish_to_all_am_prepared_cb, data);
 }
 
@@ -433,21 +433,33 @@ initial_position_cb (GeocluePosition *position,
     }
 }
 
-static gboolean
-set_requirements (EmpathyLocationManager *self)
+static void
+set_requirements (EmpathyLocationManager *self,
+    GeoclueSetRequirementsCallback callback)
 {
-  GError *error = NULL;
+  geoclue_master_client_set_requirements_async (self->priv->gc_client,
+      GEOCLUE_ACCURACY_LEVEL_COUNTRY, 0, FALSE, self->priv->resources,
+      callback, self);
+}
 
-  if (!geoclue_master_client_set_requirements (self->priv->gc_client,
-          GEOCLUE_ACCURACY_LEVEL_COUNTRY, 0, FALSE, self->priv->resources,
-          &error))
+static void
+update_resources_set_requirements_cb (GeoclueMasterClient *client,
+    GError *error,
+    gpointer userdata)
+{
+  EmpathyLocationManager *self = userdata;
+
+  if (error != NULL)
     {
       DEBUG ("set_requirements failed: %s", error->message);
       g_error_free (error);
-      return FALSE;
+      return;
     }
 
-  return TRUE;
+  geoclue_address_get_address_async (self->priv->gc_address,
+      initial_address_cb, self);
+  geoclue_position_get_position_async (self->priv->gc_position,
+      initial_position_cb, self);
 }
 
 static void
@@ -461,65 +473,112 @@ update_resources (EmpathyLocationManager *self)
   /* As per Geoclue bug #15126, using NONE results in no address
    * being found as geoclue-manual report an empty address with
    * accuracy = NONE */
-  if (!set_requirements (self))
-    return;
-
-  geoclue_address_get_address_async (self->priv->gc_address,
-      initial_address_cb, self);
-  geoclue_position_get_position_async (self->priv->gc_position,
-      initial_position_cb, self);
+  set_requirements (self, update_resources_set_requirements_cb);
 }
 
 static void
-setup_geoclue (EmpathyLocationManager *self)
+create_address_cb (GeoclueMasterClient *client,
+    GeoclueAddress *address,
+    GError *error,
+    gpointer userdata)
 {
-  GeoclueMaster *master;
-  GError *error = NULL;
+  EmpathyLocationManager *self = userdata;
 
-  DEBUG ("Setting up Geoclue");
-  master = geoclue_master_get_default ();
-  self->priv->gc_client = geoclue_master_create_client (master, NULL, &error);
-  g_object_unref (master);
-
-  if (self->priv->gc_client == NULL)
-    {
-      DEBUG ("Failed to GeoclueMasterClient: %s", error->message);
-      g_error_free (error);
-      return;
-    }
-
-  if (!set_requirements (self))
-    return;
-
-  /* Get updated when the position is changes */
-  self->priv->gc_position = geoclue_master_client_create_position (
-      self->priv->gc_client, &error);
-  if (self->priv->gc_position == NULL)
-    {
-      DEBUG ("Failed to create GeocluePosition: %s", error->message);
-      g_error_free (error);
-      return;
-    }
-
-  g_signal_connect (G_OBJECT (self->priv->gc_position), "position-changed",
-      G_CALLBACK (position_changed_cb), self);
-
-  /* Get updated when the address changes */
-  self->priv->gc_address = geoclue_master_client_create_address (
-      self->priv->gc_client, &error);
-  if (self->priv->gc_address == NULL)
+  if (error != NULL)
     {
       DEBUG ("Failed to create GeoclueAddress: %s", error->message);
       g_error_free (error);
       return;
     }
 
+  self->priv->gc_address = address;
+
   g_signal_connect (G_OBJECT (self->priv->gc_address), "address-changed",
       G_CALLBACK (address_changed_cb), self);
 
   self->priv->geoclue_is_setup = TRUE;
 }
 
+static void
+create_position_cb (GeoclueMasterClient *client,
+    GeocluePosition *position,
+    GError *error,
+    gpointer userdata)
+{
+  EmpathyLocationManager *self = userdata;
+
+  if (error != NULL)
+    {
+      DEBUG ("Failed to create GeocluePosition: %s", error->message);
+      g_error_free (error);
+      return;
+    }
+
+  self->priv->gc_position = position;
+
+  g_signal_connect (G_OBJECT (self->priv->gc_position), "position-changed",
+      G_CALLBACK (position_changed_cb), self);
+
+  /* Get updated when the address changes */
+  geoclue_master_client_create_address_async (self->priv->gc_client,
+      create_address_cb, self);
+}
+
+static void
+create_client_set_requirements_cb (GeoclueMasterClient *client,
+    GError *error,
+    gpointer userdata)
+{
+  EmpathyLocationManager *self = userdata;
+
+  if (error != NULL)
+    {
+      DEBUG ("set_requirements failed: %s", error->message);
+      g_error_free (error);
+      return;
+    }
+
+  /* Get updated when the position is changes */
+  geoclue_master_client_create_position_async (self->priv->gc_client,
+      create_position_cb, self);
+}
+
+static void
+create_client_cb (GeoclueMaster *master,
+    GeoclueMasterClient *client,
+    char *object_path,
+    GError *error,
+    gpointer userdata)
+{
+  EmpathyLocationManager *self = userdata;
+
+  if (error != NULL)
+    {
+      DEBUG ("Failed to create GeoclueMasterClient: %s", error->message);
+      g_error_free (error);
+      return;
+    }
+
+  /* @client seems be (transfer full) looking at the geoclue code; yeah for
+   * undocumented API... */
+  self->priv->gc_client = client;
+
+  set_requirements (self, create_client_set_requirements_cb);
+}
+
+static void
+setup_geoclue (EmpathyLocationManager *self)
+{
+  GeoclueMaster *master;
+
+  DEBUG ("Setting up Geoclue");
+  master = geoclue_master_get_default ();
+
+  geoclue_master_create_client_async (master, create_client_cb, self);
+
+  g_object_unref (master);
+ }
+
 static void
 publish_cb (GSettings *gsettings_loc,
             const gchar *key,
@@ -609,7 +668,7 @@ account_manager_prepared_cb (GObject *source_object,
   EmpathyLocationManager *self = user_data;
   GError *error = NULL;
 
-  if (!tp_account_manager_prepare_finish (account_manager, result, &error))
+  if (!tp_proxy_prepare_finish (account_manager, result, &error))
     {
       DEBUG ("Failed to prepare account manager: %s", error->message);
       g_error_free (error);
@@ -641,7 +700,7 @@ empathy_location_manager_init (EmpathyLocationManager *self)
   /* Setup account status callbacks */
   priv->account_manager = tp_account_manager_dup ();
 
-  tp_account_manager_prepare_async (priv->account_manager, NULL,
+  tp_proxy_prepare_async (priv->account_manager, NULL,
       account_manager_prepared_cb, self);
 
   /* Setup settings status callbacks */
