@@ -24,8 +24,6 @@
 
 #include <folks/folks.h>
 
-#include <libempathy-gtk/empathy-roster-contact.h>
-
 G_BEGIN_DECLS
 
 #define EMPATHY_ROSTER_MODEL_GROUP_TOP_GROUP _("Top Contacts")
@@ -44,9 +42,6 @@ struct _EmpathyRosterModelInterface
   GList * (* get_individuals) (EmpathyRosterModel *self);
   GList * (*get_groups_for_individual) (EmpathyRosterModel *self,
       FolksIndividual *individual);
-  GList * (*get_top_individuals) (EmpathyRosterModel *self);
-  gboolean (*contact_in_top) (EmpathyRosterModel *self,
-      EmpathyRosterContact *contact);
 };
 
 GType empathy_roster_model_get_type (void);
@@ -79,13 +74,6 @@ void empathy_roster_model_fire_groups_changed (EmpathyRosterModel *self,
     const gchar *group,
     gboolean is_member);
 
-void empathy_roster_model_fire_top_individuals_changed (
-    EmpathyRosterModel *self);
-
-void empathy_roster_model_fire_favourites_changed (EmpathyRosterModel *self,
-    FolksIndividual *individual,
-    gboolean favourite);
-
 /* Public API */
 GList * empathy_roster_model_get_individuals (EmpathyRosterModel *self);
 
@@ -93,11 +81,6 @@ GList * empathy_roster_model_get_groups_for_individual (
     EmpathyRosterModel *self,
     FolksIndividual *individual);
 
-GList * empathy_roster_model_get_top_individuals (EmpathyRosterModel *self);
-
-gboolean empathy_roster_model_contact_in_top (EmpathyRosterModel *self,
-    EmpathyRosterContact *contact);
-
 G_END_DECLS
 
 #endif /* #ifndef __EMPATHY_ROSTER_MODEL_H__*/
