@@ -304,16 +304,6 @@ individual_added (EmpathyRosterView *self,
   else
     {
       GList *groups, *l;
-      GList *tops;
-
-      tops = empathy_roster_model_get_top_individuals (self->priv->model);
-
-      if (folks_favourite_details_get_is_favourite (
-            FOLKS_FAVOURITE_DETAILS (individual)) ||
-          g_list_index (tops, individual) != -1)
-        {
-          add_to_group (self, individual, EMPATHY_ROSTER_MODEL_GROUP_TOP_GROUP);
-        }
 
       groups = empathy_roster_model_get_groups_for_individual (self->priv->model,
           individual);
@@ -494,6 +484,40 @@ individual_removed_cb (EmpathyRosterModel *model,
   individual_removed (self, individual);
 }
 
+static gboolean
+contact_in_top (EmpathyRosterView *self,
+    EmpathyRosterContact *contact)
+{
+  if (!self->priv->show_groups)
+    {
+      /* Always display top contacts in non-group mode. */
+      GList *groups;
+      FolksIndividual *individual;
+      gboolean result = FALSE;
+
+      individual = empathy_roster_contact_get_individual (contact);
+
+      groups = empathy_roster_model_get_groups_for_individual (
+          self->priv->model, individual);
+
+      if (g_list_find (groups, EMPATHY_ROSTER_MODEL_GROUP_TOP_GROUP) != NULL)
+        result = TRUE;
+
+      g_list_free (groups);
+
+      return result;
+    }
+
+  if (!tp_strdiff (empathy_roster_contact_get_group (contact),
+          EMPATHY_ROSTER_MODEL_GROUP_TOP_GROUP))
+    /* If we are displaying contacts, we only want to *always* display the
+     * RosterContact which is displayed at the top; not the ones displayed in
+     * the 'normal' group sections */
+    return TRUE;
+
+  return FALSE;
+}
+
 static gint
 compare_roster_contacts_by_alias (EmpathyRosterContact *a,
     EmpathyRosterContact *b)
@@ -517,8 +541,8 @@ compare_roster_contacts_no_group (EmpathyRosterView *self,
 {
   gboolean top_a, top_b;
 
-  top_a = empathy_roster_model_contact_in_top (self->priv->model, a);
-  top_b = empathy_roster_model_contact_in_top (self->priv->model, b);
+  top_a = contact_in_top (self, a);
+  top_b = contact_in_top (self, b);
 
   if (top_a == top_b)
     /* Both contacts are in the top of the roster (or not). Sort them
@@ -741,22 +765,8 @@ contact_should_be_displayed (EmpathyRosterView *self,
   if (self->priv->show_offline)
       return TRUE;
 
-  if (empathy_roster_model_contact_in_top (self->priv->model,
-          contact))
-    {
-      const gchar *group_name;
-
-      if (!self->priv->show_groups)
-        /* Always display favourite contacts in non-group mode. */
-        return TRUE;
-
-      group_name = empathy_roster_contact_get_group (contact);
-
-      if (!tp_strdiff (group_name, EMPATHY_ROSTER_MODEL_GROUP_TOP_GROUP))
-        /* Always display favourite contact in group mode only in the
-         * 'top group'*/
-        return TRUE;
-    }
+  if (contact_in_top (self, contact))
+    return TRUE;
 
   return empathy_roster_contact_is_online (contact);
 }
@@ -833,27 +843,6 @@ filter_list (GtkWidget *child,
   g_return_val_if_reached (FALSE);
 }
 
-/* @list: GList of EmpathyRosterContact
- *
- * Returns: %TRUE if @list contains an EmpathyRosterContact associated with
- * @individual */
-static gboolean
-individual_in_list (FolksIndividual *individual,
-    GList *list)
-{
-  GList *l;
-
-  for (l = list; l != NULL; l = g_list_next (l))
-    {
-      EmpathyRosterContact *contact = l->data;
-
-      if (empathy_roster_contact_get_individual (contact) == individual)
-        return TRUE;
-    }
-
-  return FALSE;
-}
-
 static void
 populate_view (EmpathyRosterView *self)
 {
@@ -905,69 +894,6 @@ remove_from_group (EmpathyRosterView *self,
   gtk_container_remove (GTK_CONTAINER (self), contact);
 }
 
-static void
-update_top_contacts (EmpathyRosterView *self)
-{
-  GList *tops, *l;
-  GList *to_add = NULL, *to_remove = NULL;
-  EmpathyRosterGroup *group;
-
-  if (!self->priv->show_groups)
-    {
-      egg_list_box_resort (EGG_LIST_BOX (self));
-      return;
-    }
-
-  tops = empathy_roster_model_get_top_individuals (self->priv->model);
-
-  group = g_hash_table_lookup (self->priv->roster_groups,
-      EMPATHY_ROSTER_MODEL_GROUP_TOP_GROUP);
-  if (group == NULL)
-    {
-      to_add = g_list_copy (tops);
-    }
-  else
-    {
-      GList *contacts;
-
-      contacts = empathy_roster_group_get_widgets (group);
-
-      /* Check which EmpathyRosterContact have to be removed */
-      for (l = contacts; l != NULL; l = g_list_next (l))
-        {
-          EmpathyRosterContact *contact = l->data;
-          FolksIndividual *individual;
-
-          if (empathy_roster_model_contact_in_top (self->priv->model,
-                  contact))
-            continue;
-
-          individual = empathy_roster_contact_get_individual (contact);
-
-          if (g_list_find (tops, individual) == NULL)
-            to_remove = g_list_prepend (to_remove, individual);
-        }
-
-      /* Check which EmpathyRosterContact have to be added */
-      for (l = tops; l != NULL; l = g_list_next (l))
-        {
-          FolksIndividual *individual = l->data;
-
-          if (!individual_in_list (individual, contacts))
-            to_add = g_list_prepend (to_add, individual);
-        }
-    }
-
-  for (l = to_add; l != NULL; l = g_list_next (l))
-    add_to_group (self, l->data, EMPATHY_ROSTER_MODEL_GROUP_TOP_GROUP);
-
-  for (l = to_remove; l != NULL; l = g_list_next (l))
-    remove_from_group (self, l->data, EMPATHY_ROSTER_MODEL_GROUP_TOP_GROUP);
-
-  g_list_free (to_add);
-  g_list_free (to_remove);
-}
-
 static void
 groups_changed_cb (EmpathyRosterModel *model,
     FolksIndividual *individual,
@@ -976,7 +902,10 @@ groups_changed_cb (EmpathyRosterModel *model,
     EmpathyRosterView *self)
 {
   if (!self->priv->show_groups)
-    return;
+    {
+      egg_list_box_resort (EGG_LIST_BOX (self));
+      return;
+    }
 
   if (is_member)
     {
@@ -988,45 +917,6 @@ groups_changed_cb (EmpathyRosterModel *model,
     }
 }
 
-static void
-top_individuals_changed_cb (EmpathyRosterModel *model,
-    EmpathyRosterView *self)
-{
-  update_top_contacts (self);
-}
-
-static void
-favourites_changed_cb (EmpathyRosterModel *model,
-    FolksIndividual *individual,
-    gboolean favourite,
-    EmpathyRosterView *self)
-{
-  GHashTable *contacts;
-
-  contacts = g_hash_table_lookup (self->priv->roster_contacts, individual);
-  if (contacts == NULL)
-    return;
-
-  if (self->priv->show_groups)
-    {
-      if (favourite)
-        add_to_group (self, individual, EMPATHY_ROSTER_MODEL_GROUP_TOP_GROUP);
-      else
-        remove_from_group (self, individual,
-            EMPATHY_ROSTER_MODEL_GROUP_TOP_GROUP);
-    }
-  else
-    {
-      GtkWidget *contact;
-
-      contact = g_hash_table_lookup (contacts, NO_GROUP);
-      if (contact == NULL)
-        return;
-
-      egg_list_box_child_changed (EGG_LIST_BOX (self), contact);
-    }
-}
-
 static void
 empathy_roster_view_constructed (GObject *object)
 {
@@ -1047,10 +937,6 @@ empathy_roster_view_constructed (GObject *object)
       G_CALLBACK (individual_removed_cb), self, 0);
   tp_g_signal_connect_object (self->priv->model, "groups-changed",
       G_CALLBACK (groups_changed_cb), self, 0);
-  tp_g_signal_connect_object (self->priv->model, "top-individuals-changed",
-      G_CALLBACK (top_individuals_changed_cb), self, 0);
-  tp_g_signal_connect_object (self->priv->model, "favourites-changed",
-      G_CALLBACK (favourites_changed_cb), self, 0);
 
   egg_list_box_set_sort_func (EGG_LIST_BOX (self),
       roster_view_sort, self, NULL);
